﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace VariableKeyboard
{
    public class KeyboardVariable : INotifyPropertyChanged
    {
        private string _name = "";
        private KeyboardVariableValue _value = null;
        private FindMethods _findMethod = FindMethods.Contains;
        public enum FindMethods { Contains, Whole }

        public KeyboardVariable(string name, KeyboardVariableValue value, FindMethods findMethod)
        {
            Name = name;
            Value = value;
            FindMethod = findMethod;
        }

        public string Name
        {
            get
            {
                return _name;
            }
            set
            {
                _name = value;
                NotifyPropertyChanged("Name");
            }
        }

        public KeyboardVariableValue Value
        {
            get
            {
                return _value;
            }

            set
            {
                _value = value;
                NotifyPropertyChanged("Value");
            }
        }

        public string Action
        {
            get
            {
                return Value.GetActionString();
            }
        }

        public FindMethods FindMethod
        {
            get
            {
                return _findMethod;
            }
            set
            {
                _findMethod = value;
                NotifyPropertyChanged("FindMethod");
            }
        }

        #region "KeyboardVariableValues"
        public abstract class KeyboardVariableValue
        {
            private object _defaultValue;
            public object Value;

            public object DefaultValue
            {
                get { return _defaultValue; }
            }

            public void RestoreDefaultValue()
            {
                Value = DefaultValue;
            }

            public abstract void PerformAction();

            public override string ToString()
            {
                return Value.ToString();
            }

            public virtual string GetActionString()
            {
                return "No action";
            }

            public void Initialize()
            {
                _defaultValue = Value;
            }

        }

        public class StringValue : KeyboardVariableValue
        {
            public StringValue(string value)
            {
                base.Value = value;
                base.Initialize();
            }

            public new string Value
            {
                get { return base.Value.ToString(); }
                set { base.Value = value; }
            }

            public new string DefaultValue
            {
                get { return base.DefaultValue.ToString(); }
            }

            public override void PerformAction()
            {

            }

        }

        public class IntegerValue : KeyboardVariableValue
        {
            public int IncrementValue = 0;

            public new int Value
            {
                get {return (int) base.Value; }
                set { base.Value = value; }
            }

            public new int DefaultValue
            {
                get { return (int) base.DefaultValue; }
            }

            public IntegerValue(int value)
            {
                base.Value = value;
                base.Initialize();
            }

            public IntegerValue(int value, int incValue)
            {
                base.Value = value;
                IncrementValue = incValue;
                base.Initialize();
            }

            public override void PerformAction()
            {
                Value += IncrementValue;
            }

            public override string GetActionString()
            {
                return (IncrementValue > 0) ? "Increase by " + IncrementValue : "Decrease by " + Math.Abs(IncrementValue);
            }

        }
        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }
}
